"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTuyaMainApplicationInit = exports.addTuyaMainApplicationImport = exports.withTuyaOnAndroid = void 0;
const config_plugins_1 = require("@expo/config-plugins");
const generateCode_1 = require("@expo/config-plugins/build/utils/generateCode");
// Using helpers keeps error messages unified and helps cut down on XML format changes.
const { addMetaDataItemToMainApplication, getMainApplicationOrThrow } = config_plugins_1.AndroidConfig.Manifest;
const withTuyaOnAndroid = (config, props) => {
    if (!props.androidApiKey) {
        throw new Error('You must provide an androidApiKey to use react-native-tuya. See the docs for more info: https://docs.expo.io/versions/latest/sdk/react-native-tuya/');
    }
    if (!props.androidApiSecret) {
        throw new Error('You must provide an androidApiSecret to use react-native-tuya. See the docs for more info: https://docs.expo.io/versions/latest/sdk/react-native-tuya/');
    }
    withAppBuildGradleImport(config, props);
    withProjectBuildGradleVersion(config, props);
    withTuyaMainApplication(config, props);
    return (0, config_plugins_1.withAndroidManifest)(config, async (config) => {
        // Modifiers can be async, but try to keep them fast.
        config.modResults = await setCustomConfigAsync(config, config.modResults, props);
        return config;
    });
};
exports.withTuyaOnAndroid = withTuyaOnAndroid;
// Splitting this function out of the mod makes it easier to test.
async function setCustomConfigAsync(config, androidManifest, props) {
    // Get the <application /> tag and assert if it doesn't exist.
    const mainApplication = getMainApplicationOrThrow(androidManifest);
    addMetaDataItemToMainApplication(mainApplication, 
    // value for `android:name`
    'TUYA_SMART_APPKEY', 
    // value for `android:value`
    props.androidApiKey);
    addMetaDataItemToMainApplication(mainApplication, 
    // value for `android:name`
    'TUYA_SMART_SECRET', 
    // value for `android:value`
    props.androidApiSecret);
    return androidManifest;
}
const withProjectBuildGradleVersion = (config, props) => {
    return (0, config_plugins_1.withProjectBuildGradle)(config, (config) => {
        if (config.modResults.language !== 'groovy')
            throw new Error('react-native-tuya config plugin does not support Kotlin /build.gradle yet.');
        config.modResults.contents = addTuyaMavenRepository(config.modResults.contents, props).contents;
        return config;
    });
};
function addTuyaMavenRepository(src, props) {
    const newSrc = [];
    newSrc.push('        // In case of error, make sure to add the following in the repository section');
    newSrc.push('        maven { url "https://maven-other.tuya.com/repository/maven-releases/" }');
    return (0, generateCode_1.mergeContents)({
        tag: 'react-native-tuya',
        src,
        newSrc: newSrc.join('\n'),
        anchor: /allprojects(?:\s+)?\{/,
        offset: 2,
        comment: '//',
    });
}
const withAppBuildGradleImport = (config, props) => {
    return (0, config_plugins_1.withAppBuildGradle)(config, (config) => {
        if (config.modResults.language !== 'groovy')
            throw new Error('react-native-tuya config plugin does not support Kotlin app/build.gradle yet.');
        config.modResults.contents = addPackageOptions(config.modResults.contents, props).contents;
        return config;
    });
};
function addPackageOptions(src, props) {
    const newSrc = [];
    newSrc.push(`
  packagingOptions {
    pickFirst '**/armeabi-v7a/libc++_shared.so'
    pickFirst '**/x86/libc++_shared.so'
    pickFirst '**/arm64-v8a/libc++_shared.so'
    pickFirst '**/x86_64/libc++_shared.so'
    pickFirst '**/x86/libjsc.so'
    pickFirst '**/armeabi-v7a/libjsc.so'
  }`);
    return (0, generateCode_1.mergeContents)({
        tag: 'react-native-tuya',
        src,
        newSrc: newSrc.join('\n'),
        anchor: /android(?:\s+)?\{/,
        offset: 1,
        comment: '//',
    });
}
const withTuyaMainApplication = (config, props) => {
    return (0, config_plugins_1.withMainApplication)(config, (config) => {
        if (['java'].includes(config.modResults.language)) {
            try {
                config.modResults.contents = addTuyaMainApplicationImport(config.modResults.contents).contents;
                config.modResults.contents = addTuyaMainApplicationInit(config.modResults.contents, props).contents;
            }
            catch (error) {
                if (error.code === 'ERR_NO_MATCH') {
                    throw new Error('Cannot add Tuya Library to the project\'s MainApplication because it\'s malformed. Please report this with a copy of your project MainApplication.');
                }
                throw error;
            }
        }
        else {
            throw new Error('Cannot setup Tuya Library because the MainApplication is not Java');
        }
        return config;
    });
};
function addTuyaMainApplicationImport(src) {
    return (0, generateCode_1.mergeContents)({
        tag: 'react-native-tuya-import',
        src,
        newSrc: 'import com.tuya.smart.rnsdk.core.TuyaCoreModule;',
        anchor: /import java\.util\.List;/,
        offset: 1,
        comment: '//',
    });
}
exports.addTuyaMainApplicationImport = addTuyaMainApplicationImport;
function addTuyaMainApplicationInit(src, props) {
    return (0, generateCode_1.mergeContents)({
        tag: 'react-native-tuya-init',
        src,
        newSrc: [
            '    TuyaCoreModule.Companion.initTuyaSDKWithoutOptions(this);',
        ].join('\n'),
        // anchor:
        //   / {2}UIView *rootView = [self.reactDelegate createRootViewWithBridge:bridge moduleName:@"main" initialProperties:nil];/,
        anchor: /ApplicationLifecycleDispatcher\.onApplicationCreate\(this\);/,
        offset: 1,
        comment: '//',
    });
}
exports.addTuyaMainApplicationInit = addTuyaMainApplicationInit;
